--------------------------------------------------------------------------------------------
-- Client Lua Script for EepieGeepies
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "GroupLib"

--local Apollo, ApolloTimer, GameLib, GroupLib, GridWindow

-----------------------------------------------------------------------------------------------
-- EepieGeepies Module Definition
-----------------------------------------------------------------------------------------------
local EepieGeepies = {}
EepieGeepies.version = 1.2
EepieGeepies.addMode = false
EepieGeepies.groupMembers = {}
EepieGeepies.groupMembersArray = {}
EepieGeepies.standbyMembers = {}
EepieGeepies.lootHistory = {}
EepieGeepies.exportFormat = "As bbCode"
EepieGeepies.slotMap = {
	["augment"] = "lowcost", -- 180
	["chest"] = "highcost", -- 426
	["feet"] = "mediumcost", -- 335
	["gadget"] = "lowcost", -- proc 
	["hands"] = "mediumcost", -- 335
	["head"] = "mediumcost", -- 377
	["legs"] = "highcost", -- 427
	["primary weapon"] = "mediumcost", -- 376
	["shields"] = "mediumcost", -- 360
	["shoulder"] = "lowcost", -- 264
	["itemslot-17"] = "lowcost", -- bag
	["itemslot-7"] = "lowcost", -- 261  weapon attachment
	["itemslot-9"] = "highcost", -- attunement key	
}

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Local functions
-----------------------------------------------------------------------------------------------

local function trim(s)
  return (s:gsub("^%s*(.-)%s*$", "%1"))
end

local function ConvertTimeToString(timeValue)
	return os.date("%H:%M:%S", timeValue)
end

local function FindStatusWindow(wndMain)
	return wndMain:FindChild("StatusWindow")
end

local function FindOptionsWindow(wndMain)
	return wndMain:FindChild("OptionsContainer")
end

local function ToggleTimerButton(wndMain, showStart)
	local startWindow = wndMain:FindChild("StartTimer")
	local stopWindow = wndMain:FindChild("StopTimer")
	if showStart == true then
		startWindow:Show(true)
		stopWindow:Show(false)
	else
		startWindow:Show(false)
		stopWindow:Show(true)
	end
end

local function ToggleDecayButton(wndMain, showNeeded)
	local normalWindow = wndMain:FindChild("DecayNormal")
	local neededWindow = wndMain:FindChild("DecayNeeded")
	if showNeeded == true then
		normalWindow:Show(false)
		neededWindow:Show(true)
	else
		normalWindow:Show(true)
		neededWindow:Show(false)
	end
end

local function EnableButtons(wndMain, enable)
	local addLootButton = wndMain:FindChild("AddLoot")	
	addLootButton:Enable(enable)
	--local addEPButton = wndMain:FindChild("AddEP")
	--addEPButton:Enable(enable)
end
	
local function AddGroupMember(inName)
	if inName then
		local name, msg = EPGP.FindPlayerName(inName)
		if name then
			EepieGeepies.groupMembers[name] = 1
			table.insert(EepieGeepies.groupMembersArray, name)
		end
	end
end

local function RefreshGroupMembers()
	local tempGroupMembers = {}
	if GroupLib.InGroup() then
		local groupSize = GroupLib.GetMemberCount()
		for i = 1, groupSize do
			local member = GroupLib.GetGroupMember(i)
			if member then
				local name, msg = EPGP.FindPlayerName(member.strCharacterName)
				if name then
					table.insert(tempGroupMembers, name)
				end
			end
		end
	else
		local name, msg = EPGP.FindPlayerName(GameLib.GetPlayerUnit():GetName())
		if name then
			table.insert(tempGroupMembers, name)
		end
	end
	
	local changed = false
	if #EepieGeepies.groupMembersArray ~= #tempGroupMembers then
		changed = true
	else
		table.sort(tempGroupMembers)
		
		for i=1,#tempGroupMembers do
			if tempGroupMembers[i] ~= EepieGeepies.groupMembersArray[i] then
				changed = true
			end
		end
	end	
	
	if changed == true then
		EepieGeepies.groupMembers = {}
		EepieGeepies.groupMembersArray = {}
		for i=1,#tempGroupMembers do
			AddGroupMember(tempGroupMembers[i])
		end
		
		table.sort(EepieGeepies.groupMembersArray)
	end
	
	return changed
end

local function SetInRaidValueAndSort(grid, row, col, inRaid, isStandby, priority)
	local value = ""
	local sortValue = 1
	if inRaid then
		value = "CRB_HUDAlerts:sprAlert_Check"
		sortValue = 3
	elseif isStandby then
		value = "Market:UI_Auction_Icon_TimeGreen"
		sortValue = 2
	end
	
	grid:SetCellImage(row, col, value)
	grid:SetCellSortText(row, col, string.format("%d%09.3f", sortValue, priority))
end

local function SetCellIntegerAndSort(grid, row, col, number)
	grid:SetCellData(row, col, number)
	grid:SetCellSortText(row, col, string.format("%06d", number))
end

local function SetCellFloatAndSort(grid, row, col, number)
	grid:SetCellData(row, col, string.format("%.1f", number))
	grid:SetCellSortText(row, col, string.format("%09.3f", number))
end

local function MergeGroupAndStandyby()
	local standbyNames = {}
	for name, _ in pairs(EepieGeepies.standbyMembers) do
		table.insert(standbyNames, name)
	end
	
	table.sort(standbyNames)
	
	local mergedArray = {}
	for _, name in ipairs(EepieGeepies.groupMembersArray) do
		table.insert(mergedArray, name)
	end
	
	for _, name in ipairs(standbyNames) do
		table.insert(mergedArray, name)
	end
	
	return mergedArray
end

local function ProcessGroupTimer(wndMain, currentTime, refreshGroup)
	local redraw = false
	if EepieGeepies.nextTimerTime then
		
		if refreshGroup == true then
			RefreshGroupMembers()
		end
		
		if EepieGeepies.groupMembersArray and #EepieGeepies.groupMembersArray > 0 then
			if currentTime >= EepieGeepies.nextTimerTime then
				if (currentTime - EepieGeepies.nextTimerTime) > 3600 then
					-- been offline for over an hour, so assume the timer should be turned off
					EepieGeepies.lastTimerTime = nil
					EepieGeepies.nextTimerTime = nil
					EepieGeepies.standbyMembers = {}
					StopTimer(wndMain, "Group timer automatically disabled due to being offline for a long time")
				else
					EepieGeepies.lastTimerTime = EepieGeepies.nextTimerTime
					EepieGeepies.nextTimerTime = EepieGeepies.nextTimerTime + EPGP.GetBonusSeconds()
					
					local groupString = ""
					local mergedArray = MergeGroupAndStandyby()
					for i=1,#mergedArray do
						if i > 1 then
							groupString = groupString .. ","
						end
						
						groupString = groupString .. mergedArray[i]
						local msg = EPGP.AddEp(mergedArray[i], EPGP.GetBonusEp())
						if msg and string.len(msg) > 0 then
							EepieGeepies.AddStatusText(msg)
						end
					end

					EepieGeepies.AddStatusText("Group timer payment of " .. EPGP.GetBonusEp() .. " made to group [" .. groupString .. "]")
				end
				
				redraw = true
			end
		end
	end
	
	return redraw
end

local function SendStandbyReminder(name)
	local charNames = EPGP.GetPlayerCharacterNames(name)
	for lowername, name in pairs(charNames) do
		local strText = name .. " STANDBY CHECK: Reply to this whisper with epgp to remain on standby"
		local tInput = ChatSystemLib.SplitInput(strText)
		EepieGeepies.channel:Send(tInput.strMessage)
	end
end

local function RemoveStandbyPlayer(name)
	EepieGeepies.standbyMembers[name] = nil
	EepieGeepies.AddStatusText("Removed account " .. name .. " from the standby list")
end

local function ProcessStandbyTimer(currentTime)
	local redraw = false
	if EepieGeepies.nextStandbyCheck then
		if EepieGeepies.lastStandbyCheck and currentTime > EepieGeepies.lastStandbyCheck + EPGP.GetStandbyTimeout() then
			local removeList = {}
			for name, when in pairs(EepieGeepies.standbyMembers) do
				if when < EepieGeepies.lastStandbyCheck then
					table.insert(removeList, name)
				end
			end
			
			for _, name in ipairs(removeList) do
				RemoveStandbyPlayer(name)
				redraw = true
			end
			
			EepieGeepies.lastStandbyCheck = nil
		elseif currentTime > EepieGeepies.nextStandbyCheck then
			for name, when in pairs(EepieGeepies.standbyMembers) do
				if when <= EepieGeepies.nextStandbyCheck then
					SendStandbyReminder(name)
				end
			end
			
			EepieGeepies.lastStandbyCheck = EepieGeepies.nextStandbyCheck
			EepieGeepies.nextStandbyCheck = EepieGeepies.nextStandbyCheck + EPGP.GetStandbyCheckFrequency()
		end
	end
	
	return redraw
end

local function GetPlayerSelectorNames()
	local playerNameArray = EPGP.GetPlayerNames()

	local names = {}	
	for i=1,#EepieGeepies.groupMembersArray do
		table.insert(names, EepieGeepies.groupMembersArray[i])
	end
	
	for i=1,#playerNameArray do
		if not EepieGeepies.groupMembers[playerNameArray[i]] then
			table.insert(names, playerNameArray[i])
		end
	end
	
	return names
end

local function SetPlayerDropToggle(window, name)
	local toggle = window:FindChild("PlayerDropToggle")
	toggle:Enable(true)
	toggle:SetText(name)
end

local function FillPlayerSelector(window, selectedName)
	local names = GetPlayerSelectorNames()
	local selectorList = window:FindChild("PlayerSelector"):FindChild("PlayerList")
	local selectedRow = 0
	local scrollPosition = selectorList:GetVScrollPos()
	selectorList:DeleteAll()
	
	for i, name in ipairs(names) do
		if selectedName == name then
			selectedRow = i
		end
		
		selectorList:AddRow(name)
	end
	
	selectorList:SetCurrentRow(selectedRow)
	selectorList:SetVScrollPos(scrollPosition)
	SetPlayerDropToggle(window, selectedName)
end

local function SetFormatDropToggle(window, name)
	local toggle = window:FindChild("FormatDropToggle")
	toggle:Enable(true)
	toggle:SetText(name)
end

local function FillFormatSelector(window, selectedName)
	local names = {"As bbCode", "As CSV", "As Enjin", "As HTML", "As JSON", "Database"}
	local selectorList = window:FindChild("FormatSelector"):FindChild("FormatList")
	local selectedRow = 0
	local scrollPosition = selectorList:GetVScrollPos()
	selectorList:DeleteAll()
	
	for i, name in ipairs(names) do
		if selectedName == name then
			selectedRow = i
		end
		
		selectorList:AddRow(name)
	end
	
	selectorList:SetCurrentRow(selectedRow)
	selectorList:SetVScrollPos(scrollPosition)
	SetFormatDropToggle(window, selectedName)
end

local function SetAddEPGroupButton(wndAddEP, isChecked)
	if isChecked == true then
		wndAddEP:FindChild("GroupButton"):SetCheck(true)
		wndAddEP:FindChild("PlayerButton"):SetCheck(false)
		wndAddEP:FindChild("PlayerSelector"):Enable(false)
	else
		wndAddEP:FindChild("GroupButton"):SetCheck(false)
		wndAddEP:FindChild("PlayerButton"):SetCheck(true)
		wndAddEP:FindChild("PlayerSelector"):Enable(true)
	end
end

local function SetAddEPAddButton(wndAddEP)
	wndAddEP:FindChild("AddButton"):SetCheck(true)
	wndAddEP:FindChild("RemoveButton"):SetCheck(false)
end

local function EnableMainButtons(wndMain, enable)
	wndMain:FindChild("AddLoot"):Enable(enable)
	wndMain:FindChild("AddEP"):Enable(enable)
	wndMain:FindChild("StartTimer"):Enable(enable)
	wndMain:FindChild("StopTimer"):Enable(enable)
	wndMain:FindChild("DecayNormal"):Enable(enable)
	wndMain:FindChild("DecayNeeded"):Enable(enable)
	EepieGeepies.addMode = not enable
end

local function SetSliderPctValue(editBox, value)
	if value then	
		local value = math.floor(math.abs(value) + 0.5)
		editBox:SetText(string.format("%d%%", value))
	end
end

local function SetSliderValue(editBox, value)
	if value then	
		local value = math.floor(math.abs(value) / 10 + 0.5) * 10
		editBox:SetText(string.format("%d", value))
	end
end

local function SetSliderBarAndText(name, wndParent, value)
	wndParent:FindChild(name .. "SliderBar"):SetValue(value)
	SetSliderValue(wndParent:FindChild(name .. "EditBox"), value)
end

local function SetSliderBarAndPctText(name, wndParent, value)
	wndParent:FindChild(name .. "SliderBar"):SetValue(value)
	SetSliderPctValue(wndParent:FindChild(name .. "EditBox"), value)
end

local function GetSliderValue(slider)
	return math.floor(math.abs(slider:GetValue()) / 10 + 0.5) * 10
end

local function GetSlotType(inSlotName, inSlotNumber)
	local slotType = nil
	if inSlotName and string.len(inSlotName) > 0 then
		slotType = EepieGeepies.slotMap[string.lower(inSlotName)]
	else
		slotType = EepieGeepies.slotMap["itemslot-" .. tostring(inSlotNumber)]
	end
	
	return slotType
end

local function SetAddEPSliderValue(wndAddEP, fNewValue)
	local value = fNewValue
	local editBox = wndAddEP:FindChild("EPEditBox")
	if not fNewValue then
		value = tonumber(editBox:GetText())
	end
	
	local value = math.abs(value)
	if wndAddEP:FindChild("RemoveButton"):IsChecked() then
		value = value * -1
	end
	
	editBox:SetText(string.format("%d", value))
end

local function ToggleOptionsImportButton(wndOpt)
	local format = wndOpt:FindChild("FormatDropToggle"):GetText()
	local visible = false
	if format == "Database" then
		visible = true
	end
	
	wndOpt:FindChild("OptionsImport"):Show(visible)
end

local function InitialiseOptions(wndOpt)
	FillPlayerSelector(wndOpt, EepieGeepies.focusPlayer)
	FillFormatSelector(wndOpt, EepieGeepies.exportFormat)
	ToggleOptionsImportButton(wndOpt)
	
	local value = EPGP.GetBonusEp()
	SetSliderBarAndText("TimerEP", wndOpt, value)

	local value = EPGP.GetBonusSeconds()
	SetSliderBarAndText("Secs", wndOpt, value)

	local value = EPGP.GetMinEp()
	SetSliderBarAndText("MinEP", wndOpt, value)

	local value = EPGP.GetBaseGp()
	SetSliderBarAndText("BaseGP", wndOpt, value)

	local value = EPGP.GetDecayPercentage() * 100
	SetSliderBarAndPctText("Decay", wndOpt, value)

	local value = EPGP.GetMaxEp()
	SetSliderBarAndText("MaxEP", wndOpt, value)

	local value = EPGP.GetSlotWeight("LowCost") * 100
	SetSliderBarAndPctText("LowCost", wndOpt, value)

	local value = EPGP.GetSlotWeight("MediumCost") * 100
	SetSliderBarAndPctText("MediumCost", wndOpt, value)

	local value = EPGP.GetSlotWeight("HighCost") * 100
	SetSliderBarAndPctText("HighCost", wndOpt, value)

	local value = EPGP.GetQualityWeight("Normal") * 100
	SetSliderBarAndPctText("Normal", wndOpt, value)

	local value = EPGP.GetQualityWeight("Epic") * 100
	SetSliderBarAndPctText("Epic", wndOpt, value)

	local value = EPGP.GetQualityWeight("Legendary") * 100
	SetSliderBarAndPctText("Legendary", wndOpt, value)

	local value = EPGP.GetItemLevelOffset()
	SetSliderBarAndText("ItemOffset", wndOpt, value)
end

local function UpdateOptions(wndMain)
	local value = wndMain:FindChild("FormatDropToggle"):GetText()
	EepieGeepies.exportFormat = value
	
	local value = GetSliderValue(wndMain:FindChild("TimerEPSliderBar"))
	local msg = EPGP.SetBonusEp(value)
	EepieGeepies.AddStatusText(msg)

	local value = GetSliderValue(wndMain:FindChild("SecsSliderBar"))
	local msg = EPGP.SetBonusSeconds(value)
	EepieGeepies.AddStatusText(msg)

	local value = GetSliderValue(wndMain:FindChild("MinEPSliderBar"))
	local msg = EPGP.SetMinEp(value)
	EepieGeepies.AddStatusText(msg)

	local value = GetSliderValue(wndMain:FindChild("BaseGPSliderBar"))
	local msg = EPGP.SetBaseGp(value)
	EepieGeepies.AddStatusText(msg)

	local value = wndMain:FindChild("DecaySliderBar"):GetValue()
	local msg = EPGP.SetDecayPercentage(value / 100)
	EepieGeepies.AddStatusText(msg)

	local value = GetSliderValue(wndMain:FindChild("MaxEPSliderBar"))
	local msg = EPGP.SetMaxEp(value)
	EepieGeepies.AddStatusText(msg)

	local value = wndMain:FindChild("LowCostSliderBar"):GetValue()
	local msg = EPGP.SetSlotWeight("LowCost", value / 100)
	EepieGeepies.AddStatusText(msg)

	local value = wndMain:FindChild("MediumCostSliderBar"):GetValue()
	local msg = EPGP.SetSlotWeight("MediumCost", value / 100)
	EepieGeepies.AddStatusText(msg)

	local value = wndMain:FindChild("HighCostSliderBar"):GetValue()
	local msg = EPGP.SetSlotWeight("HighCost", value / 100)
	EepieGeepies.AddStatusText(msg)

	local value = wndMain:FindChild("NormalSliderBar"):GetValue()
	local msg = EPGP.SetQualityWeight("Normal", value / 100)
	EepieGeepies.AddStatusText(msg)

	local value = wndMain:FindChild("EpicSliderBar"):GetValue()
	local msg = EPGP.SetQualityWeight("Epic", value / 100)
	EepieGeepies.AddStatusText(msg)

	local value = wndMain:FindChild("LegendarySliderBar"):GetValue()
	local msg = EPGP.SetQualityWeight("Legendary", value / 100)
	EepieGeepies.AddStatusText(msg)

	local value = GetSliderValue(wndMain:FindChild("ItemOffsetSliderBar"))
	local msg = EPGP.SetItemLevelOffset(value)
	EepieGeepies.AddStatusText(msg)
end

local function ShowAuditMessages()
	local auditMsgs = EPGP.GetAuditLogString()
	EepieGeepies.AddStatusText(auditMsgs, true)
end

local function CloseOptionsWindow(wndMain)
	local optionsWindow = FindOptionsWindow(wndMain)
	optionsWindow:Close()
	UpdateOptions(wndMain)
end

local function AddLootToHistory(characterName, itemName, itemLevel, slotName, slotNumber, itemQuality, playerName)
	if characterName and itemName and slotName and itemQuality then
		local slotType = GetSlotType(slotName, slotNumber)
		if slotType then
			local loot = {}
			loot.characterName = characterName
			loot.itemName = itemName
			loot.itemLevel = itemLevel
			loot.slotName = slotType
			loot.itemQuality = itemQuality
			loot.playerName = playerName
			
			table.insert(EepieGeepies.lootHistory, 1, loot)
			
			while #EepieGeepies.lootHistory > 100 do
				table.remove(EepieGeepies.lootHistory)
			end
		end
	end
end

local function DrawLootHistory(wndAddGP)
	local grid = wndAddGP:FindChild("LootGrid")
	grid:DeleteAll()
	
	for count, item in ipairs(EepieGeepies.lootHistory) do
		if item.slotName and item.itemLevel then
			grid:AddRow("Row" .. count)
			
			grid:SetCellData(count, 1, item.characterName)
			grid:SetCellData(count, 2, item.itemName)
			grid:SetCellData(count, 3, item.itemQuality)
			if item.playerName then
				grid:SetCellData(count, 4, item.playerName)
			else
				grid:SetCellData(count, 4, "")
			end
			
			grid:SetCellData(count, 5, count)
			grid:SetCellData(count, 6, item.slotName)
			grid:SetCellData(count, 7, item.itemLevel)
		end
	end
	
	grid:SetSortColumn(5, true)
end

local function CloseAddEPWindow(wndAddEP, wndMain)
	wndAddEP:Close()
	EnableMainButtons(wndMain, true)
	wndMain:FindChild("Grid"):Invoke()
	EepieGeepies.DrawMainGrid(wndMain, true)
end

local function SetAddEPError(wndAddEP, msg)
	local errorFrame = wndAddEP:FindChild("ErrorFrame")
	if not msg or string.len(msg) < 1 then
		errorFrame:Show(false)
	else
		errorFrame:FindChild("ErrorWindow"):SetText(msg)
		errorFrame:Show(true)
	end
end

local function ValidateAddEP(wndAddEP)
	local ret = true
	local msg = ""
	if wndAddEP:FindChild("PlayerButton"):IsChecked() then
		local player = wndAddEP:FindChild("PlayerDropToggle"):GetText()
		if not player or string.len(player) < 1 then
			msg = "You must choose a player when in single player mode"
			ret = false
		end
	end
	
	SetAddEPError(wndAddEP, msg)
	return ret
end

local function AddEPAction(wndAddEP)
	local ep = tonumber(wndAddEP:FindChild("EPEditBox"):GetText())
	local reason = wndAddEP:FindChild("ReasonEditBox"):GetText()
	if wndAddEP:FindChild("GroupButton"):IsChecked() then
		for i=1,#EepieGeepies.groupMembersArray do
			msg = EPGP.AddEp(EepieGeepies.groupMembersArray[i], ep, reason)
			EepieGeepies.AddStatusText(msg)			
		end
	else
		local playerName = wndAddEP:FindChild("PlayerDropToggle"):GetText()
		msg = EPGP.AddEp(playerName, ep, reason)
		EepieGeepies.AddStatusText(msg)
	end
end

local function CloseAddGPWindow(wndAddGP, wndMain)
	wndAddGP:Close()
	EnableMainButtons(wndMain, true)
	wndMain:FindChild("Grid"):Invoke()
	EepieGeepies.DrawMainGrid(wndMain, true)
end

local function SetSlotButtons(wndAddGP, inSlotName, inSlotNumber)
	local slotCost = GetSlotType(inSlotName, inSlotNumber)
	if not slotCost then
		slotCost = "lowcost"
	end
	
	wndAddGP:FindChild("HighCostSlot"):SetCheck(false)
	wndAddGP:FindChild("MediumCostSlot"):SetCheck(false)
	wndAddGP:FindChild("LowCostSlot"):SetCheck(false)
	
	if slotCost == "highcost" then
		wndAddGP:FindChild("HighCostSlot"):SetCheck(true)
	elseif slotCost == "mediumcost" then
		wndAddGP:FindChild("MediumCostSlot"):SetCheck(true)
	else
		wndAddGP:FindChild("LowCostSlot"):SetCheck(true)
	end
end

local function SetItemLevel(wndAddGP, inLevel)
	local itemLevel = 1000
	if inLevel then
		itemLevel = inLevel
	end
	
	wndAddGP:FindChild("ItemLevelEditBox"):SetText(itemLevel)
	wndAddGP:FindChild("ItemLevelSliderBar"):SetValue(itemLevel)
end

local function SetRarityButtons(wndAddGP, inQuality)
	local quality = "normal"
	if inQuality and string.len(inQuality) > 0 then
		quality = string.lower(inQuality)
	end
	
	wndAddGP:FindChild("LegendaryQuality"):SetCheck(false)
	wndAddGP:FindChild("EpicQuality"):SetCheck(false)
	wndAddGP:FindChild("NormalQuality"):SetCheck(false)
	
	if quality == "legendary" then
		wndAddGP:FindChild("LegendaryQuality"):SetCheck(true)
	elseif quality == "epic" or quality == "superb" then
		wndAddGP:FindChild("EpicQuality"):SetCheck(true)
	else
		wndAddGP:FindChild("NormalQuality"):SetCheck(true)
	end
end

local function InitialiseAddGP(wndMain, wndAddGP)
	CloseOptionsWindow(wndMain)

	EepieGeepies.lootPlayer = EepieGeepies.focusPlayer
	
	SetSlotButtons(wndAddGP, "Head", 0)
	SetRarityButtons(wndAddGP, "Epic")
	wndAddGP:FindChild("MainSpec"):SetCheck(true)
	wndAddGP:FindChild("OffSpec"):SetCheck(false)

	wndAddGP:FindChild("ItemNameEditBox"):SetText("")
	
	DrawLootHistory(wndAddGP)
end

local function SetAddGPMsg(wndAddGP, doAdd)
	local isBasic = wndAddGP:FindChild("BasicButton"):IsChecked()
	local itemName = wndAddGP:FindChild("ItemNameEditBox"):GetText()
	local msg = "Add"
	if not itemName or string.len(itemName) < 1 then
		itemName = "Unknown item"
	end
	local lootPlayer = EepieGeepies.lootPlayer
	if not lootPlayer then
		lootPlayer = "UNKNOWN"
	end
	

	if isBasic == true then
		local itemValue = tonumber(wndAddGP:FindChild("ItemValueEditBox"):GetText())
		if not itemValue then
			itemValue = 0
		end
		
		itemValue = math.floor(itemValue)
		
		msg = msg .. " " .. itemName .. " to " .. lootPlayer .. " for cost " .. itemValue
		wndAddGP:FindChild("LootMsgEditBox"):SetText(msg)

		if doAdd == true then
			local _, msg = EPGP.AddBasicGp(EepieGeepies.lootPlayer, itemName, itemValue)
			EepieGeepies.AddStatusText(msg)
		end
	else
		local mainSpec = true
		local slot = "HighCost"
		local quality = "Legendary"
		if wndAddGP:FindChild("MainSpec"):IsChecked() then
			msg = msg .. " main spec loot "
		else
			msg = msg .. " off spec loot "
			mainSpec = false
		end
		
		if itemName and string.len(itemName) > 0 then
			msg = msg .. itemName
		else
			msg = msg .. "Unknown item"
		end
	
		if wndAddGP:FindChild("HighCostSlot"):IsChecked() then
			msg = msg .. " in high cost slot"
			slot = "HighCost"
		elseif wndAddGP:FindChild("MediumCostSlot"):IsChecked() then
			msg = msg .. " in medium cost slot"
			slot = "MediumCost"
		else
			msg = msg .. " in low cost slot"
			slot = "LowCost"
		end
	
		if wndAddGP:FindChild("LegendaryQuality"):IsChecked() then
			msg = msg .. " with legendary quality"
			quality = "Legendary"
		elseif wndAddGP:FindChild("EpicQuality"):IsChecked() then
			msg = msg .. " with epic quality"
			quality = "Epic"
		else
			msg = msg .. " with normal quality"
			quality = "Normal"
		end
		
		local itemLevel = wndAddGP:FindChild("ItemLevelSliderBar"):GetValue()
		msg = msg .. " to account " .. lootPlayer .. " for cost " .. EPGP.CalculateGearValue(slot, quality, itemLevel, mainSpec)
	
		wndAddGP:FindChild("LootMsgEditBox"):SetText(msg)

		if doAdd == true then
			local _, msg = EPGP.AddGp(EepieGeepies.lootPlayer, itemName, slot, quality, itemLevel, mainSpec)
			EepieGeepies.AddStatusText(msg)
		end
	end
end

local function StopTimer(wndMain, msg)
	ToggleTimerButton(wndMain, true)
	if msg then
		EepieGeepies.AddStatusText(msg)	
	elseif EepieGeepies.lastTimerTime then
		EepieGeepies.AddStatusText("Group timer stopped.  Last payment was at " .. ConvertTimeToString(EepieGeepies.lastTimerTime))
	else
		EepieGeepies.AddStatusText("Group timer stopped.  No payments made.")
	end

	EepieGeepies.nextTimerTime = nil
	EepieGeepies.lastTimerTime = nil
	EepieGeepies.nextStandbyCheck = nil
	EepieGeepies.lastStandbyCheck = nil
	EepieGeepies.standbyMembers = {}
end

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function EepieGeepies:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function EepieGeepies:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		--EepieGeepies.TooltipAddon,
	}
	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	EPGP.new()
end
 
function EepieGeepies:OnDependencyError(strDep, strError)
	local ret = false
	
    --if strDep == EepieGeepies.TooltipAddon then
    --    local tReplacements = Apollo.GetReplacement(strDep)
    --    if #tReplacements == 1 then
	--        EepieGeepies.TooltipAddon = tReplacements[1]
    --        ret = true
    --    end
    --end

    return ret
end

-----------------------------------------------------------------------------------------------
-- EepieGeepies OnLoad
-----------------------------------------------------------------------------------------------
function EepieGeepies:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("EepieGeepies.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- EepieGeepies OnDocLoaded
-----------------------------------------------------------------------------------------------
function EepieGeepies:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "EepieGeepiesForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end

	    self.wndMain:Show(false, true)
		EnableButtons(self.wndMain, false)
		
		local optionsWindow = FindOptionsWindow(self.wndMain)
		local optionsTab = optionsWindow:FindChild("OptionsTab")
		optionsTab:FindChild("PlayerDropToggle"):AttachWindow(optionsTab:FindChild("PlayerSelectorList"))
		optionsTab:FindChild("FormatDropToggle"):AttachWindow(optionsTab:FindChild("FormatSelectorList"))
		optionsTab:AttachTab(optionsWindow:FindChild("MoreOptionsTab"), false)
		
		EepieGeepies.statusTextWnd = self.wndMain:FindChild("StatusText")

	    self.wndAddEP = self.wndMain:FindChild("AddEPContainer")
		self.wndAddEP:FindChild("PlayerDropToggle"):AttachWindow(self.wndAddEP:FindChild("PlayerSelectorList"))

	    self.wndAddGP = self.wndMain:FindChild("AddGPContainer")
		
		ShowAuditMessages()
		
		if EepieGeepies.location then
			self.wndMain:MoveToLocation(EepieGeepies.location)
			EepieGeepies.location = nil
		end
		
		ToggleDecayButton(self.wndMain, EPGP.IsDecayNeeded())
		ToggleTimerButton(self.wndMain, not EepieGeepies.nextTimerTime)
		
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("epgp", "OnEepieGeepiesOn", self)

		self.timer = ApolloTimer.Create(5.000, true, "OnTimer", self)
		Apollo.RegisterEventHandler("LootAssigned",	"OnLootAssigned", self)
		Apollo.RegisterEventHandler("LootRollWon", "OnLootRollWon", self)
		Apollo.RegisterEventHandler("ItemLink", "OnItemLink", self)
		Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
		Apollo.RegisterEventHandler("ChatTellFailed", "OnChatTellFailed", self)
		
		
		EepieGeepies.channel = nil
		for _, channelCurrent in pairs(ChatSystemLib.GetChannels()) do
			if channelCurrent:GetType() == ChatSystemLib.ChatChannel_Whisper then
				EepieGeepies.channel = channelCurrent
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- EepieGeepies Functions
-----------------------------------------------------------------------------------------------

function EepieGeepies:OnChatMessage(channelCurrent, tMessage)
	-- tMessage has bAutoResponse, bGM, bSelf, strSender, strRealmName, nPresenceState, arMessageSegments, unitSource, bShowChatBubble, bCrossFaction, nReportId

	-- arMessageSegments is an array of tables.  Each table representsa part of the message + the formatting for that segment.
	-- This allows us to signal font (alien text for example) changes mid stream.
	-- local example = arMessageSegments[1]
	-- example.strText is the text
	-- example.bAlien == true if alien font set
	-- example.bRolePlay == true if this is rolePlay Text.  RolePlay text should only show up for people in roleplay mode, and non roleplay text should only show up for people outside it.

	-- to use: 	{#}toggles alien on {*}toggles rp on. Alien is still on {!}resets all format codes.


	-- There will be a lot of chat messages, particularly for combat log.  If you make your own chat log module, you will want to batch
	-- up several at a time and only process lines you expect to see.
	
	if channelCurrent and channelCurrent:GetType() == ChatSystemLib.ChatChannel_Whisper then

		--tQueuedMessage.eChannelType = channelCurrent:GetType()
		--tQueuedMessage.strChannelName = channelCurrent:GetName()
		---tQueuedMessage.strChannelCommand = channelCurrent:GetCommand()
		if tMessage and tMessage.arMessageSegments and #tMessage.arMessageSegments > 0 and tMessage.strSender then
			local inStr = trim(tMessage.arMessageSegments[1].strText)
			if inStr and string.len(inStr) > 3 then
				local playerName = EPGP.FindPlayerName(tMessage.strSender)
				if playerName then
					local startStr = string.lower(string.sub(inStr, 1, 4))
					if startStr == "epgp" then
						if not EepieGeepies.groupMembers[playerName] then
							if not EepieGeepies.standbyMembers[playerName] then
								EepieGeepies.AddStatusText("Account " .. playerName .. " added to standby list")
								EepieGeepies.DrawMainGrid(self.wndMain, true)
							else
								EepieGeepies.AddStatusText("Account " .. playerName .. " refreshed standby status")
							end
							
							EepieGeepies.standbyMembers[playerName] = os.time()
						end
					elseif startStr == "-epg" then
						EepieGeepies.AddStatusText("Account " .. playerName .. " removed from standby list")
						EepieGeepies.standbyMembers[playerName] = nil
						EepieGeepies.DrawMainGrid(self.wndMain, true)
					end
				end
			end
		end
	end
end

function EepieGeepies:OnChatTellFailed(channel, strCharacterTo)
	--EepieGeepies.Debug(Apollo.GetString("CRB_Whisper_Error") .. " " .. Apollo.GetString("CRB_Whisper_Error_Reason"))
end

function EepieGeepies:OnLootAssigned(item, looterName)
	AddLootToHistory(looterName, item:GetName(), item:GetItemPower(), item:GetSlotName(), item:GetSlot(), item:GetItemQuality(), EPGP.FindPlayerName(looterName))
end

function EepieGeepies:OnLootRollWon(item, looterName, isNeed)
	AddLootToHistory(looterName, item:GetName(), item:GetItemPower(), item:GetSlotName(), item:GetSlot(), item:GetItemQuality(), EPGP.FindPlayerName(looterName))
end


function EepieGeepies.AddStatusText(txt, dontShowTime)
	if EepieGeepies.statusTextWnd and txt then
		local msg = tostring(txt)
		if string.len(msg) > 0 then
			local timeString = ""
			if dontShowTime ~= true then
				timeString = ConvertTimeToString(os.time()) .. " "
			end
			
			local oldTxt = EepieGeepies.statusTextWnd:GetText()
			EepieGeepies.statusTextWnd:SetText(timeString .. msg .. "\n" .. oldTxt)
		end
	end
end

function EepieGeepies.Debug(txt)
	EepieGeepies.AddStatusText(txt, true)
end

function EepieGeepies.DrawMainGrid(wndMain, redraw)
	local changed = RefreshGroupMembers()
	
	if redraw == true or changed == true then
		local grid = wndMain:FindChild("Grid")
		grid:DeleteAll()
		
		local playerNames = EPGP.GetPlayerNamesByPriority()
		for count, name in ipairs(playerNames) do
			grid:AddRow("Row" .. count)
			
			local player, msg = EPGP.GetPlayer(name)
			if not player then
				grid:SetCellData(count, 1, msg)
			else
				grid:SetCellData(count, 1, name)
				SetCellIntegerAndSort(grid, count, 2, EPGP.GetPlayerEp(player))
				SetCellIntegerAndSort(grid, count, 3, EPGP.GetPlayerGp(player))
				local priority = EPGP.GetPlayerPriority(player)
				SetCellFloatAndSort(grid, count, 4, priority)
				
				local inRaid = false
				local isStandby = false
				local playerName = EPGP.FindPlayerName(name)
				if EepieGeepies.groupMembers[playerName] then
					inRaid = true
				elseif EepieGeepies.standbyMembers[playerName] then
					isStandby = true
				end
				
				SetInRaidValueAndSort(grid, count, 5, inRaid, isStandby, priority)
				grid:SetCellData(count, 6, EPGP.GetPlayerCharacterNamesAsString(player))
			end
		end
		
		grid:SetSortColumn(5, false)
		
		if EepieGeepies.focusPlayer then
			for i=1,grid:GetRowCount() do
				local name = grid:GetCellText(i, 1)
				if name and name == EepieGeepies.focusPlayer then
					grid:SelectCell(i, 1)
				end
			end
			
			EnableButtons(wndMain, true)
		else
			EnableButtons(wndMain, false)
		end
	end
end

-- on SlashCommand "/epgp"
function EepieGeepies:OnEepieGeepiesOn()
	EepieGeepies.DrawMainGrid(self.wndMain, true)
	self.wndMain:Invoke() -- show the window
end

-- on timer
function EepieGeepies:OnTimer()
	local refreshGroup = true
	local redraw = false
	if self.wndMain:IsVisible() then
		refreshGroup = false
	end
	
	local currentTime = os.time()
	redraw = ProcessGroupTimer(self.wndMain, currentTime, refreshGroup) or redraw
	redraw = ProcessStandbyTimer(currentTime) or redraw
	
	if redraw == true then
		EepieGeepies.DrawMainGrid(self.wndMain, redraw)
	end
end

-- on variables save
function EepieGeepies:OnSave(eLevel)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account then
        return nil
    end

	local saveTable = {}
	saveTable.version = EepieGeepies.version
	saveTable.lootHistory = EepieGeepies.lootHistory
	saveTable.exportFormat = EepieGeepies.exportFormat
	saveTable.database = EPGP.GetDatabase()
	saveTable.nextTimerTime = EepieGeepies.nextTimerTime
	saveTable.lastTimerTime = EepieGeepies.lastTimerTime
	saveTable.nextStandbyCheck = EepieGeepies.nextStandbyCheck
	saveTable.lastStandbyCheck = EepieGeepies.lastStandbyCheck
	saveTable.standbyMembers = EepieGeepies.standbyMembers

	if self.wndMain then
		local location = self.wndMain:GetLocation()
		if location then
			if location.ToTable then
				saveTable.location = location:ToTable()
			end
		end
	end
	
    return saveTable
end

-- on variables restore
function EepieGeepies:OnRestore(eLevel, saveTable)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account then
        return nil
    end

	if saveTable.lootHistory and saveTable.database then
		EepieGeepies.lootHistory = saveTable.lootHistory
		EPGP.SetDatabase(saveTable.database)
		if saveTable.location then
			EepieGeepies.location = WindowLocation.new(saveTable.location)
		end
		
		if saveTable.exportFormat then
			EepieGeepies.exportFormat = saveTable.exportFormat
		end
		
		EepieGeepies.nextTimerTime = saveTable.nextTimerTime
		EepieGeepies.lastTimerTime = saveTable.lastTimerTime
		if saveTable.standbyMembers then
			EepieGeepies.nextStandbyCheck = saveTable.nextStandbyCheck
			EepieGeepies.lastStandbyCheck = saveTable.lastStandbyCheck
			EepieGeepies.standbyMembers = saveTable.standbyMembers
		end
	end
end

function EepieGeepies:OnItemLink(item)
	if item and self.wndAddGP:IsVisible() then
		local name = item:GetName()
		local itemPower = item:GetItemPower()
		local itemRarity = item:GetItemQuality()
		local itemSlotName = item:GetSlotName()
		local itemSlotNumber = item:GetSlot()
		local slotType = GetSlotType(itemSlotName, itemSlotNumber)
		
		if name and itemPower and itemRarity and slotType then
			SetSlotButtons(self.wndAddGP, itemSlot, itemSlotNumber)
			SetRarityButtons(self.wndAddGP, itemRarity)		
			self.wndAddGP:FindChild("ItemNameEditBox"):SetText(name)
			SetItemLevel(self.wndAddGP, itemPower)
			SetAddGPMsg(self.wndAddGP, false)
		end
	end
end

-----------------------------------------------------------------------------------------------
-- EepieGeepiesForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function EepieGeepies:OnOK()
	CloseAddEPWindow(self.wndAddEP, self.wndMain)
	CloseAddGPWindow(self.wndAddGP, self.wndMain)
	self.wndMain:Close() -- hide the window
end

-- when a grid item is selected
function EepieGeepies:OnGridSelChanged(wndHandler, wndControl, nRow, nCol)
	if nRow > 0 then
		EepieGeepies.focusPlayer = wndControl:GetCellText(nRow, 1)
		EnableButtons(self.wndMain, true)
		
		local optionsWindow = FindOptionsWindow(self.wndMain)
		if optionsWindow:IsVisible() then
			SetPlayerDropToggle(optionsWindow, EepieGeepies.focusPlayer)
		end
	else
		EnableButtons(self.wndMain, false)
		EepieGeepies.focusPlayer = nil
	end
end

-- when the StartTimer button is clicked
function EepieGeepies:OnStartTimer()
	ToggleTimerButton(self.wndMain, false)
	
	local currentTime = os.time()
	EepieGeepies.nextTimerTime = currentTime + EPGP.GetBonusSeconds()
	EepieGeepies.lastTimerTime = nil
	EepieGeepies.nextStandbyCheck = currentTime + EPGP.GetStandbyCheckFrequency()
	EepieGeepies.lastStandbyCheck = nil
	EepieGeepies.AddStatusText("Group timer started.  First payment will be at " .. ConvertTimeToString(EepieGeepies.nextTimerTime))
end

-- when the StartTimer button is clicked
function EepieGeepies:OnStopTimer()
	StopTimer(self.wndMain)
end

function EepieGeepies:OnGridMouseEnter(wndHandler, wndControl, x, y)
	wndControl:Enable(true)
end

function EepieGeepies:OnGridMouseExit(wndHandler, wndControl, x, y)
	wndControl:Enable(false)
end

-- when the OK button is clicked
function EepieGeepies:OnConfig()
	if EepieGeepies.addMode ~= true then
		local statusWindow = FindStatusWindow(self.wndMain)
		local optionsWindow = FindOptionsWindow(self.wndMain)
		if optionsWindow:IsVisible() then
			CloseOptionsWindow(self.wndMain)
			statusWindow:Close()
		else
			InitialiseOptions(optionsWindow)
			optionsWindow:Invoke()
			statusWindow:Invoke()
		end
	else
		CloseOptionsWindow(self.wndMain)

		local statusWindow = FindStatusWindow(self.wndMain)
		if statusWindow:IsVisible() then
			statusWindow:Close()
		else
			statusWindow:Invoke()
		end
	end
end

function EepieGeepies:OnStatusButtonClose()
	local statusWindow = FindStatusWindow(self.wndMain)
	statusWindow:Close()
end

function EepieGeepies:OnAddEP()
	CloseOptionsWindow(self.wndMain)

	self.wndMain:FindChild("Grid"):Close()
	FillPlayerSelector(self.wndAddEP, EepieGeepies.focusPlayer)
	SetAddEPGroupButton(self.wndAddEP, true)
	SetAddEPAddButton(self.wndAddEP)
	SetAddEPSliderValue(self.wndAddEP)
	SetAddEPError(self.wndAddEP, "")
	EnableMainButtons(self.wndMain, false)
	self.wndAddEP:Invoke()
end

function EepieGeepies:OnAddEPPlayerSelected(wndHandler, wndControl, nRow)
	local name = wndControl:GetCellText(nRow, 1)
	if name then
		self.wndAddEP:FindChild("PlayerDropToggle"):SetText(name)
	end
end

function EepieGeepies:OnOptionsPlayerSelected(wndHandler, wndControl, nRow)
	local name = wndControl:GetCellText(nRow, 1)
	if name then
		FindOptionsWindow(self.wndMain):FindChild("PlayerDropToggle"):SetText(name)
	end
end

function EepieGeepies:OnAddEPAdd()
	SetAddEPSliderValue(self.wndAddEP)
end

function EepieGeepies:OnAddEPRemove()
	SetAddEPSliderValue(self.wndAddEP)
end

function EepieGeepies:OnOptionsExport()
	local format = self.wndMain:FindChild("FormatDropToggle"):GetText()
	local data = ""
	if format == "As bbCode" then
		data = EPGP.ExportDatabaseAsBBCode()
	elseif format == "As CSV" then
		data = EPGP.ExportDatabaseAsCSV()
	elseif format == "As Enjin" then
		data = EPGP.ExportDatabaseAsEnjin()
	elseif format == "As HTML" then
		data = EPGP.ExportDatabaseAsHTML()
	elseif format == "As JSON" then
		data = EPGP.ExportDatabaseAsJSON()
	elseif format == "Database" then
		data = EPGP.ExportDatabase()
	end
	
	self.wndMain:FindChild("ImportExportText"):SetText(data)
end

function EepieGeepies:OnOptionsImport( wndHandler, wndControl, eMouseButton )
	local data = self.wndMain:FindChild("ImportExportText"):GetText()
	EPGP.ImportDatabase(data)
	EepieGeepies.groupMembers = {}
	EepieGeepies.groupMembersArray = {}
	EepieGeepies.DrawMainGrid(self.wndMain, true)
	ToggleDecayButton(self.wndMain, EPGP.IsDecayNeeded())
end

function EepieGeepies:OnOptionsAddPlayer()
	local player = self.wndMain:FindChild("OptionsPlayerText")
	local playerName = player:GetText()
	if playerName and string.len(playerName) > 0 then
		local ret, msg = EPGP.AddPlayer(playerName)
		EepieGeepies.AddStatusText(msg)
		local ret, msg = EPGP.AddCharacter(playerName, playerName)
		EepieGeepies.AddStatusText(msg)
		EepieGeepies.DrawMainGrid(self.wndMain, true)
	end
	
	player:SetText("")
end

function EepieGeepies:OnOptionsDeletePlayer()
	local player = self.wndMain:FindChild("OptionsPlayerText")
	local playerName = player:GetText()
	if playerName and string.len(playerName) > 0 then
		local ret, msg = EPGP.DeletePlayer(playerName)
		EepieGeepies.AddStatusText(msg)
		EepieGeepies.DrawMainGrid(self.wndMain, true)
	end
	
	player:SetText("")
end

function EepieGeepies:OnOptionsAddCharacter()
	local optionsWindow = FindOptionsWindow(self.wndMain)
	local player = optionsWindow:FindChild("PlayerDropToggle")
	local playerName = player:GetText()
	local character = optionsWindow:FindChild("OptionsCharacterText")
	local characterName = character:GetText()
	if playerName and characterName and string.len(playerName) > 0 and string.len(characterName) > 0 then
		local ret, msg = EPGP.AddCharacter(playerName, characterName)
		EepieGeepies.AddStatusText(msg)
		EepieGeepies.DrawMainGrid(self.wndMain, true)
	end
	
	character:SetText("")
end

function EepieGeepies:OnOptionsDeleteCharacter()
	local optionsWindow = FindOptionsWindow(self.wndMain)
	local player = optionsWindow:FindChild("PlayerDropToggle")
	local playerName = player:GetText()
	local character = optionsWindow:FindChild("OptionsCharacterText")
	local characterName = character:GetText()
	if playerName and characterName and string.len(playerName) > 0 and string.len(characterName) > 0 then
		local ret, msg = EPGP.DeleteCharacter(playerName, characterName)
		EepieGeepies.AddStatusText(msg)
		EepieGeepies.DrawMainGrid(self.wndMain, true)
	end
	
	character:SetText("")
end

function EepieGeepies:OnOptionsCloseClick()
	CloseOptionsWindow(self.wndMain)
end

function EepieGeepies:OnAddGPOK()
	SetAddGPMsg(self.wndAddGP, true)
	CloseAddGPWindow(self.wndAddGP, self.wndMain)
end

function EepieGeepies:OnAddGPCancel()
	CloseAddGPWindow(self.wndAddGP, self.wndMain)
end

function EepieGeepies:OnAddLoot()
	self.wndMain:FindChild("Grid"):Close()
	InitialiseAddGP(self.wndMain, self.wndAddGP)
	SetAddGPMsg(self.wndAddGP)
	EnableMainButtons(self.wndMain, false)
	self.wndAddGP:Invoke()
end

function EepieGeepies:OnLootMsgChange()
	SetAddGPMsg(self.wndAddGP)
end

function EepieGeepies:OnLootGridSelChanged(wndHandler, wndControl, nRow, nCol)
	if nRow > 0 then
		local lootPlayer = wndControl:GetCellText(nRow, 4)
		if not lootPlayer or string.len(lootPlayer) < 1 then
			lootPlayer = EepieGeepies.focusPlayer
		end
		
		EepieGeepies.lootPlayer = lootPlayer
		
		local lootName = wndControl:GetCellText(nRow, 2)
		self.wndAddGP:FindChild("ItemNameEditBox"):SetText(lootName)
		
		local lootRarity = wndControl:GetCellText(nRow, 3)
		SetRarityButtons(self.wndAddGP, lootRarity)
		
		local lootSlot = wndControl:GetCellText(nRow, 6)
		SetSlotButtons(self.wndAddGP, lootSlot, 0)
		
		local lootItemLevel = wndControl:GetCellText(nRow, 7)
		SetItemLevel(self.wndAddGP, lootItemLevel)
	else
		EepieGeepies.lootPlayer = EepieGeepies.focusPlayer
		SetRarityButtons(self.wndAddGP)
		SetSlotButtons(self.wndAddGP)
		SetItemLevel(self.wndAddGP)
	end
	
	SetAddGPMsg(self.wndAddGP, false)
end

function EepieGeepies:OnTimerEPSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("TimerEPEditBox")
	SetSliderValue(textBox, fNewValue)
end

function EepieGeepies:OnTimerSecsSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("SecsEditBox")
	SetSliderValue(textBox, fNewValue)
end

function EepieGeepies:OnMinEPSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("MinEPEditBox")
	SetSliderValue(textBox, fNewValue)
end

function EepieGeepies:OnMaxEPSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("MaxEPEditBox")
	SetSliderValue(textBox, fNewValue)
end

function EepieGeepies:OnBaseGPSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("BaseGPEditBox")
	SetSliderValue(textBox, fNewValue)
end

function EepieGeepies:OnDecaySliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("DecayEditBox")
	SetSliderPctValue(textBox, fNewValue)
end

function EepieGeepies:OnDecay( wndHandler, wndControl, eMouseButton )
	local msg = EPGP.DecayDatabase()
	EepieGeepies.AddStatusText(msg)
	EepieGeepies.DrawMainGrid(self.wndMain, true)
end

function EepieGeepies:OnItemLevelSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndAddGP:FindChild("ItemLevelEditBox")
	SetSliderValue(textBox, fNewValue)
	SetAddGPMsg(self.wndAddGP, false)
end

function EepieGeepies:OnOptionsTogglePlayer( wndHandler, wndControl, eMouseButton )
end

function EepieGeepies:OnNormalSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("NormalEditBox")
	SetSliderPctValue(textBox, fNewValue)
end

function EepieGeepies:OnEpicSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("EpicEditBox")
	SetSliderPctValue(textBox, fNewValue)
end

function EepieGeepies:OnLegendarySliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("LegendaryEditBox")
	SetSliderPctValue(textBox, fNewValue)
end

function EepieGeepies:OnLowCostSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("LowCostEditBox")
	SetSliderPctValue(textBox, fNewValue)
end

function EepieGeepies:OnMediumCostSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("MediumCostEditBox")
	SetSliderPctValue(textBox, fNewValue)
end

function EepieGeepies:OnHighCostSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("HighCostEditBox")
	SetSliderPctValue(textBox, fNewValue)
end

function EepieGeepies:OnItemOffsetSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local textBox = self.wndMain:FindChild("ItemOffsetEditBox")
	SetSliderValue(textBox, fNewValue)
end

function EepieGeepies:OnOptionsFormatSelected(wndHandler, wndControl, nRow)
	local format = wndControl:GetCellText(nRow, 1)
	if format then
		local wndOpt = FindOptionsWindow(self.wndMain)
		wndOpt:FindChild("FormatDropToggle"):SetText(format)
		ToggleOptionsImportButton(wndOpt, format)
	end
end

function EepieGeepies:OnAddGPBasicMode( wndHandler, wndControl, eMouseButton )
	local advanced = true
	if wndControl:IsChecked() then
		advanced = false
	end
	
	self.wndAddGP:FindChild("AdvancedFrame"):Show(advanced)
	self.wndAddGP:FindChild("BasicFrame"):Show(not advanced)
	SetAddGPMsg(self.wndAddGP, false)
end

function EepieGeepies:OnItemChanged( wndHandler, wndControl, strText )
	SetAddGPMsg(self.wndAddGP, false)
end

---------------------------------------------------------------------------------------------------
-- AddEP Functions
---------------------------------------------------------------------------------------------------

function EepieGeepies:OnAddEPWholeGroup( wndHandler, wndControl, eMouseButton )
	SetAddEPGroupButton(self.wndAddEP, true)
end

function EepieGeepies:OnAddEPSinglePlayer( wndHandler, wndControl, eMouseButton )
	SetAddEPGroupButton(self.wndAddEP, false)
end

function EepieGeepies:OnAddEPTogglePlayer( wndHandler, wndControl, eMouseButton )
end

function EepieGeepies:OnAddEPSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	SetAddEPSliderValue(self.wndAddEP, fNewValue)
end

function EepieGeepies:OnAddEPOK()
	if ValidateAddEP(self.wndAddEP) == true then
		AddEPAction(self.wndAddEP)
		CloseAddEPWindow(self.wndAddEP, self.wndMain)
	end
end

function EepieGeepies:OnAddEPCancel()
	CloseAddEPWindow(self.wndAddEP, self.wndMain)
end

-----------------------------------------------------------------------------------------------
-- EepieGeepies Instance
-----------------------------------------------------------------------------------------------
local EepieGeepiesInst = EepieGeepies:new()
EepieGeepiesInst:Init()
